/*
 * Probe that tracks when the program starts
 *
 * usage: insmod probe_execve.ko process=<process_name>
 * parameter <process_name> is the name of the process we want to monitor,
 * default process is bash
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/kprobes.h>
#include <linux/ktime.h>
#include <linux/limits.h>

#include "probe_execve.h"

static char process_name[NAME_MAX];
//module_param_string(process, process_name, NAME_MAX, S_IRUGO);
//MODULE_PARM_DESC(process, "Process to monitor with probe");

/* defined in mod_calc_stacks.h, implemented in mod_calc_stacks.c */
void signalProgramStart(char*,long long);

/* function called when user writes process name in /proc/profiled_process */
void signalProgramNameProbeExecve(char* p_name) {
	strlcpy(process_name, p_name, strlen(p_name));	
	printk (KERN_INFO "process_name in probe_execve.c set to %s\n", process_name);
}

/*
 * Jumper probe for sys_execve function.
 * Mirror principle enables access to arguments of the probed function
 * from the probe handler.
 */

/* Proxy function having the same arguments as the actual sys_execve function */
long jsys_execve(const char __user *name,
		const char __user *const __user *argv,
		const char __user *const __user *envp, struct pt_regs *regs) {
	
	if (strlen(process_name) > 0 && strstr(name,process_name)) {
		signalProgramStart(process_name, (long long) ktime_to_ns(ktime_get()));		
		//printk(KERN_INFO "process %s started at time: %lld\n", process_name, (long long) ktime_to_ns(ktime_get()));
	}

	/* Always end with a call to jprobe_return(). */
	jprobe_return();
	return 0;
}

static struct jprobe my_jprobe = {
	.entry			= jsys_execve,
	.kp = {
		.symbol_name	= "sys_execve",
	},
};

static int __init jprobe_init(void) {
	int ret;

	ret = register_jprobe(&my_jprobe);
	if (ret < 0) {
		printk(KERN_INFO "register_jprobe failed, returned %d\n", ret);
		return -1;
	}
	
	printk(KERN_INFO "Planted jprobe at %p, handler addr %p\n", my_jprobe.kp.addr, my_jprobe.entry);

	return 0;
}

static void __exit jprobe_exit(void) {
	unregister_jprobe(&my_jprobe);
	printk(KERN_INFO "jprobe at %p unregistered\n", my_jprobe.kp.addr);
}


/* Expose functions to other modules */
EXPORT_SYMBOL(signalProgramNameProbeExecve); 

module_init(jprobe_init)
module_exit(jprobe_exit)

MODULE_LICENSE("GPL");
