#!/usr/bin/ruby

require 'pp'

unless ARGV.length == 2 
	puts "Remember: ./script_generator.rb <inputfile> <outputFile>"
	exit
end

input = ARGV[0]
output = ARGV[1]

colors = ["#0000FF","#7FFF00","#FF7F50","#008B8B","#B8860B","#A9A9A9","#8FBC8F","#FF1493","#00BFFF","#F0E68C"]


threadColors = [ ["MainThread","#00FFFF"],
		 ["Collector","#A52A2A"],
		 ["Organizer","#006400"],
		 ["FinalizerThread", "#FFD700"],
		 ["TimerThread","#191970"],
		 ["ControllerThread","#8B008B"],
		 ["CompilationThread","#00FA9A"] ]


outputFile = File.open(output + ".gnuplot", "w")

outputFile.puts("set terminal postscript eps enhanced color font \"Arial,12\"")
outputFile.puts("set output \"" + output + ".eps\"")

inputFile = IO.readlines(input)

title = inputFile[0].delete("\n")

outputFile.puts("set title \"" + title + "\"")
outputFile.puts("set grid")
outputFile.puts("set xlabel \"Parallelism\"")
outputFile.puts("set ylabel \"Runtime (in seconds)\"")
outputFile.puts("set key outside right center")
outputFile.puts("set style fill solid 1.0 noborder")

maxValX = inputFile[1].delete("\n")

outputFile.puts("set xrange [-" + maxValX + ":" + maxValX + "]")

outputFile.write("set xtics (")

for i in ((maxValX.to_i) * -1)..(maxValX.to_i)
	outputFile.write( "\"" + i.abs.to_s + "\" " + i.to_s  )
	
	if i < maxValX.to_i
		outputFile.write(",")
	else
		outputFile.write(")\n")
	end
end


class ThreadRecord
	attr_accessor :name, :pid, :criticality, :runtime, :parallelism, :concurrentThreads

	def initialize()
		@concurrentThreads = Array.new
	end
end

class ConcurrentThreadRecord
	attr_accessor :pid, :criticality
end

all_threads = Array.new()

#mapping array => pid, threadname
mapping_array = Hash.new 

i = 2
while i < inputFile.size
	line = inputFile[i]

	if line.chomp.size == 0
		i += 1
	end

	if line.chomp.size > 0 and not line.start_with?("c:")
		temp = line.split(",")
	
		thread = ThreadRecord.new
		thread.name = temp[0]
		thread.pid = temp[1].to_i
		thread.criticality = temp[2].to_i
		thread.runtime = temp[3].to_i
		thread.parallelism = thread.runtime.to_f / thread.criticality.to_f
	
		mapping_array[thread.pid] = thread.name

		i += 1
		while inputFile[i] && inputFile[i].start_with?("c:")  && i < inputFile.size
			concThread = ConcurrentThreadRecord.new
			temp2 = inputFile[i].split(",")
			concThread.pid = temp2[0].gsub("c:","").to_i
			concThread.criticality = temp2[1].gsub("\n","").to_i
			
			if concThread.criticality > 0
				thread.concurrentThreads.push(concThread)
			end
			i += 1
		end

		# sort data
		thread.concurrentThreads.sort! { |a,b| b.criticality <=> a.criticality}	

		all_threads.push(thread)
	end
end


#debug tool
#all_threads.each do |thread|
#	puts "name: " + thread.name
#	puts "criticality: " + thread.criticality.to_s
#	puts "runtime: " + thread.runtime.to_s
#	puts "parallelism: " + thread.parallelism.to_s
#end

# calculate max y-value

max_y_range = { "avrora" => 12,
		        "jython" => 12,
		        "luindex" => 3,
		        "lusearch" => 7,
		        "pmd" => 4,
		        "sunflow" => 7,
		        "xalan" => 6,
		        "antlr" => 4,
		        "bloat" => 7,
		        "eclipse" => 40,
		        "fop" => 3,
				"PJBB2005" => 5}

maxY = 0
all_threads.each do |thread|
	maxY += thread.criticality
end
maxY = (maxY.to_f / 1000000000).ceil

benchmark = title.split(" ")[0]

#if maxY > max_y_range[benchmark]
	outputFile.puts("set yrange [0:" + maxY.to_s + "]")
#else
#	outputFile.puts("set yrange [0:" + max_y_range[benchmark].to_s + "]")
#end

# code for drawing boxes
base = 0
box_counter = 1
threadOrder = Array.new
color = ""

while all_threads.length > 0 do

	temp_max = 0
	temp_pos = 0
	
	# zoeken thread met hoogste parallelism in array	
	for i in 0..(all_threads.length-1)
		if all_threads[i].parallelism > temp_max
			temp_max = all_threads[i].parallelism
			temp_pos = i
		end
	end
	
	# kijk of gekozen thread service thread is, zoja kies vaste kleur voor deze thread
	threadColors.each do |threadColor|
		if all_threads[temp_pos].name.include? threadColor[0] 
			color = threadColor[1]
		end
	end
	
	# gekozen thread is geen service thread, kies kleur uit lijst
	if color == ""
		thread_idx = all_threads[temp_pos].name.gsub( /[a-z]+/i , '').to_i
		color = colors[( thread_idx ) % colors.length]
	end

	# gnuplot code voor het tekenen van de box voor de thread
	#outputFile.puts("set object " + box_counter.to_s + " rect from -1," + base.to_s + " to 1," + (base+(all_threads[temp_pos].criticality).to_f/1000000000).to_s + " fc rgb \"" + color + "\"")
	outputFile.puts("set object " + box_counter.to_s + " rect from " + (all_threads[temp_pos].parallelism * -1).to_s  + " ," + base.to_s + " to " + all_threads[temp_pos].parallelism.to_s + "," + (base+(all_threads[temp_pos].criticality).to_f/1000000000).to_s + " fc rgb \"" + color + "\"")
	box_counter += 1

	#outputFile.puts("set object " + box_counter.to_s + " rect from -1," + base.to_s + " to 0," + (base+(all_threads[temp_pos].criticality).to_f/1000000000).to_s + " fc rgb \"" + color + "\"")
	#box_counter += 1

	# tekenen boxes concurrent threads
	horizontal_pos = 1;	
	all_threads[temp_pos].concurrentThreads.each do |concThread|
		# find name thread
		if not mapping_array.include? concThread.pid
			break
		else
			concThreadName = mapping_array[concThread.pid]
		end
		
		# find color 
		color = ""
		# kijk of gekozen thread service thread is, zoja kies vaste kleur voor deze thread
		threadColors.each do |threadColor|
			if concThreadName.eql? threadColor[0] 
				color = threadColor[1]
			end
		end
	
		# gekozen thread is geen service thread, kies kleur uit lijst
		if color == ""
			thread_idx = concThreadName.gsub( /[a-z]+/i , '').to_i
			color = colors[( thread_idx ) % colors.length]
		end

		offset = concThread.criticality.to_f/(all_threads[temp_pos].criticality).to_f
		#outputFile.puts("set object " + box_counter.to_s + " rect from " + (horizontal_pos).to_s + "," + base.to_s + " to " + (horizontal_pos + offset).to_s + "," + (base+(all_threads[temp_pos].criticality).to_f/1000000000).to_s + " fc rgb \"" + color + "\"")
		#outputFile.puts("set object " + box_counter.to_s + " fillstyle pattern 2 border")
		#box_counter += 1


		outputFile.puts("set object " + box_counter.to_s + " rect from " + ((horizontal_pos + offset)*-1).to_s + "," + base.to_s + " to " + (horizontal_pos*-1).to_s + "," + (base+(all_threads[temp_pos].criticality).to_f/1000000000).to_s + " fc rgb \"" + color + "\"")
		#outputFile.puts("set object " + box_counter.to_s + " fillstyle pattern 2 border")
		box_counter += 1
		

		horizontal_pos += offset
	end


	# basis op verticale as verschuiven voor volgende thread
	base = base + (all_threads[temp_pos].criticality).to_f/1000000000
	
	# orde van threads bijhouden voor legende
	threadOrder.push all_threads[temp_pos].name
	
	# verwijder thread uit lijst van threads
	all_threads.delete_at(temp_pos)

	# kleur terug op null zetten voor volgende iteratie
	color = ""
end

# gnuplot code voor legende
outputFile.write("plot ")

color = ""
for i in 0..(threadOrder.length-1)

	threadColors.each do |threadColor|
		if threadOrder[threadOrder.length - 1 - i].include? threadColor[0] 
			color = threadColor[1]
		end
	end
	
	if color == ""
		thread_idx = threadOrder[threadOrder.length - 1 - i].gsub( /[a-z]+/i , '').to_i
		color = colors[( thread_idx ) % colors.length]
	end

	outputFile.write( "-1 with boxes title \"" + threadOrder[threadOrder.length - 1 - i] + "\" lc rgb \"" + color + "\"")

	if i < threadOrder.length-1
		outputFile.write(", ")
	end	
	
	color = ""
end

outputFile.close
# inputFile.close
