#!/usr/bin/ruby

require 'fileutils.rb'

unless ARGV.length == 3
	puts "Remember: ./runDacapo.rb <times_min_heap> <num_GC_threads> <num_app_threads>"
	exit
end

#bms_new = ["avrora" , "jython" , "luindex" , "lusearch" , "pmd" , "sunflow" , "xalan"] 
#bms_old = ["antlr" , "bloat" , "eclipse" , "fop"] 

bms_new = ["xalan","lusearch","pmd","sunflow"]
bms_old = []

min_heap = { "avrora" => 50,
	     "jython" => 120,
	     "luindex" => 30,
	     "lusearch" => 40,
	     "pmd" => 50,
	     "sunflow" => 60,
	     "xalan" => 60,
	     "antlr" => 30,
	     "bloat" => 100,
	     "eclipse" => 100,
	     "fop" => 60 }

times_min_heap = ARGV[0].to_f	   
num_GC_threads = ARGV[1].to_i
num_app_threads = ARGV[2].to_i

results_dir = "/scratch/kdubois/results/"
scripts_dir = "/scratch/kdubois/critical_threads/scripts/"
 
dirName = times_min_heap.round().to_s + "x_minHeap_" + num_GC_threads.to_s + "_GCs_" + num_app_threads.to_s + "_app_openjdk"
output_dir = results_dir + dirName 

FileUtils.mkdir_p output_dir

puts `echo java > /proc/profiled_process`
puts `echo #{num_app_threads} > /scratch/kdubois/numCores`

# run the benchmarks + parse thread names
bms_new.each do |benchmark|
	heap_size = (min_heap[benchmark] * times_min_heap).round()
	
	title = benchmark + "_heap" + heap_size.to_s + "MB_GC-threads" + num_GC_threads.to_s + "_app-threads" + num_app_threads.to_s
	
	puts `echo clear_stack > /proc/signal_action ; echo init_stack > /proc/signal_action`
	puts `echo #{title} > /proc/program_name_output`
		
	puts `taskset -c 0-7,16-23 /scratch/kdubois/openjdk/jdk6/build/linux-amd64/bin/java -Xms#{heap_size}M -Xmx#{heap_size}M  -server -Xss128K -XX:+UseParallelGC -XX:+UseParallelOldGC -XX:+UseBiasedLocking -XX:+AggressiveOpts -XX:ParallelGCThreads=#{num_GC_threads} -Dprobes=StopJIT -Dprobe.stopjit.iteration=13 -cp /scratch/kdubois/jikes/bin/probes/probes.jar:/scratch/kdubois/jikes/benchmarks/dacapo/dacapo-kristof.jar Harness -c probe.DacapoBachCallback -n 16 #{benchmark} > #{output_dir}/#{benchmark}.log 2>&1`
	
	puts `#{scripts_dir}bm_output_parser_openjdk.rb #{output_dir}/#{benchmark}.log #{output_dir}/#{benchmark}.threadNames`
end  

bms_old.each do |benchmark|
	heap_size = (min_heap[benchmark] * times_min_heap).round()
	
	title = benchmark + "_heap" + heap_size.to_s + "MB_GC-threads" + num_GC_threads.to_s + "_app-threads" + num_app_threads.to_s
	
	puts `echo clear_stack > /proc/signal_action ; echo init_stack > /proc/signal_action`
	puts `echo #{title} > /proc/program_name_output`
		
	puts `taskset -c 0-7,16-23 /scratch/kdubois/openjdk/jdk6/build/linux-amd64/bin/java -Xms#{heap_size}M -Xmx#{heap_size}M  -server -Xss128K -XX:+UseParallelGC -XX:+UseParallelOldGC -XX:+UseBiasedLocking -XX:+AggressiveOpts -XX:ParallelGCThreads=#{num_GC_threads} -Dprobes=StopJIT -Dprobe.stopjit.iteration=13 -cp /scratch/kdubois/jikes/bin/probes/probes.jar:/scratch/kdubois/jikes/benchmarks/dacapo/dacapo-kristof-old.jar Harness -c probe.Dacapo2006Callback -n 16 #{benchmark} > #{output_dir}/#{benchmark}.log 2>&1`

	puts `#{scripts_dir}bm_output_parser_openjdk.rb #{output_dir}/#{benchmark}.log #{output_dir}/#{benchmark}.threadNames`
end	

# parse results
puts `#{scripts_dir}results_log_parser_futexes.rb #{output_dir}`

# format data for gnuplot-script generator 
Dir.chdir(output_dir) do
	Dir.glob("*.futexes").each do |file|
		puts `#{scripts_dir}input_plot_generator_futexes.rb #{file} #{output_dir}`
	end
end

# invoke gnuplot-script generator
#Dir.chdir(output_dir) do
#	Dir.glob("*.data.formatted").each do |file|
#		temp = file.gsub(".data.formatted","")
#		puts `#{scripts_dir}gnuplot_script_generator_openjdk.rb #{output_dir}/#{file} #{output_dir}/#{temp}`
#	end
#end

# invoke gnuplot
#Dir.chdir(output_dir) do
#	Dir.glob("*.gnuplot").each do |file|
#		puts `gnuplot #{output_dir}/#{file}`
#	end
#end
   	   
